from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.db import models
import managers

class URL(models.Model):
    """ A URL that exists in order to alias an object to a different location. """

    class Meta(object):
        verbose_name = 'URL'

    location = models.CharField(max_length=255, unique=True)

    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    related_object = generic.GenericForeignKey()

    # Custom manager for getting related items easily
    objects = managers.URLManager()

    def get_related_url(self):
        if hasattr(self.related_object, 'get_absolute_url'):
            return self.related_object.get_absolute_url()

    def get_absolute_url(self):
        return self.location

    def __unicode__(self):
        return self.location




from django.http import HttpResponseRedirect, Http404
from django.conf import settings
from django.core.urlresolvers import resolve, Resolver404
from exceptions import IndexError
from models import URL
import operator, re

class AliasFallbackMiddleware(object):
    def process_response(self, request, response):
        if response.status_code != 404:
            return response

        try:
            if hasattr(settings, 'ALIASES_MAP_ARGS') and settings.ALIASES_MAP_ARGS is False:
                alias = URL.objects.get(location=request.path_info)
            else:
                # I'm not a fan of raw SQL queries, but this should work in any SQL server (I think). If you know how to do
                # this in django's ORM, let me know. If this doesn't work and/or you don't need this functionality, you can
                # always set settings.ALIASES_MAP_ARGS to False and avoid this.

                alias = URL.objects.raw('SELECT * FROM aliases_url WHERE "%s" LIKE CONCAT(location, "%%") ORDER BY LENGTH(location) LIMIT 1', [request.path_info])

                try:
                    alias = alias[0]
                except IndexError:
                    alias = None

            if alias is not None:
                if alias.get_related_url():
                    try:
                        match = resolve(alias.get_related_url())
                    except Resolver404:
                        match = None
                else:
                    match = None

                # When we have extra data in the request, we pass it as args to the view
                if len(alias.location) < len(request.path_info):
                    additional_args = re.sub('\/$', '', request.path_info)
                    additional_args = additional_args[len(alias.location):].split('/')

                    # This is a tuple, which is immutable. Convert it to a list.
                    match.args = list(match.args)

                    match.args.extend(additional_args)

                if match:
                    return match.func(request, *match.args, **match.kwargs)
                else:
                    return HttpResponseRedirect(alias.get_related_url())
            else:
                raise Http404('The specified URL mapping does not exist.')

        except URL.DoesNotExist:
            return response

        except:
            if settings.DEBUG:
                raise

            return response




from django.db.models import Manager
from django.contrib import contenttypes

class URLManager(Manager):
    def related_to(self, obj):
        content_type = contenttypes.models.ContentType.objects.get_for_model(obj.__class__)
        return super(URLManager, self).get_query_set().filter(content_type=content_type, object_id=obj.pk)




#!/usr/bin/env python
from distutils.core import setup
import os

parent_directory = os.path.abspath(os.path.dirname(__file__))

meta_files = {
    'README.md': None,
    'CLASSIFIERS.txt': None,
}

for filename in meta_files:
    try:
        current_file = open(os.path.join(parent_directory, filename))
        meta_files[filename] = current_file.read()
        current_file.close()
    except IOError:
        raise IOError('{0} not found.'.format(filename))

classifiers = meta_files['CLASSIFIERS.txt'].split('\n')
classifiers.remove('')

setup(name='django-aliases',
      version='0.1',
      description='Some useful tools for "aliasing" objects in django.',
      long_description=meta_files['README.md'],
      classifiers=classifiers,
      author='Brandon R. Stoner',
      author_email='monokrome@limpidtech.com',
      url='http://github.com/LimpidTech/django-aliases',
      packages=['aliases'],
      keywords = 'web django aliases shortcuts',
)

